@@ -46,13 +46,15 @@ from typing import (
 )
 
 import psutil
+import tomli as tomllib
 
 from .. import command_arguments, dataclasses_merge, find_directories, identifiers
 from ..filesystem import expand_global_root, expand_relative_path
 from ..find_directories import (
-    CONFIGURATION_FILE,
     get_relative_local_root,
+    JSON_CONFIGURATION_FILE,
     LOCAL_CONFIGURATION_FILE,
+    TOML_CONFIGURATION_FILE,
 )
 from . import (
     exceptions,
@@ -113,6 +115,7 @@ class PartialConfiguration:
     dot_pyre_directory: Optional[Path] = None
     enable_readonly_analysis: Optional[bool] = None
     enable_strict_override_check: Optional[bool] = None
+    enable_strict_any_check: Optional[bool] = None
     enable_unawaited_awaitable_analysis: Optional[bool] = None
     excludes: Sequence[str] = field(
         default_factory=list,
@@ -130,6 +133,7 @@ class PartialConfiguration:
     isolation_prefix: Optional[str] = None
     logger: Optional[str] = None
     number_of_workers: Optional[int] = None
+    max_number_of_workers: Optional[int] = None
     oncall: Optional[str] = None
     other_critical_files: Sequence[str] = field(
         default_factory=list,
@@ -168,7 +172,6 @@ class PartialConfiguration:
     unwatched_dependency: Optional[unwatched.UnwatchedDependency] = None
     use_buck2: Optional[bool] = None
     version_hash: Optional[str] = None
-    use_errpy_parser: Optional[bool] = None
 
     @staticmethod
     def _get_extra_keys() -> Set[str]:
@@ -215,6 +218,7 @@ class PartialConfiguration:
             isolation_prefix=arguments.isolation_prefix,
             logger=arguments.logger,
             number_of_workers=arguments.number_of_workers,
+            max_number_of_workers=arguments.max_number_of_workers,
             oncall=None,
             other_critical_files=[],
             pysa_version_hash=None,
@@ -250,7 +254,7 @@ class PartialConfiguration:
         )
 
     @staticmethod
-    def from_string(contents: str) -> "PartialConfiguration":
+    def from_dict(configuration_json: Dict[str, Any]) -> "PartialConfiguration":
         def is_list_of_string(elements: object) -> bool:
             return isinstance(elements, list) and all(
                 isinstance(element, str) for element in elements
@@ -337,8 +341,6 @@ class PartialConfiguration:
             return search_path
 
         try:
-            configuration_json = json.loads(contents)
-
             dot_pyre_directory = ensure_option_type(
                 configuration_json, "dot_pyre_directory", str
             )
@@ -437,6 +439,9 @@ class PartialConfiguration:
                 enable_strict_override_check=ensure_option_type(
                     configuration_json, "enable_strict_override_check", bool
                 ),
+                enable_strict_any_check=ensure_option_type(
+                    configuration_json, "enable_strict_any_check", bool
+                ),
                 enable_unawaited_awaitable_analysis=ensure_option_type(
                     configuration_json, "enable_unawaited_awaitable_analysis", bool
                 ),
@@ -460,6 +465,9 @@ class PartialConfiguration:
                 number_of_workers=ensure_option_type(
                     configuration_json, "workers", int
                 ),
+                max_number_of_workers=ensure_option_type(
+                    configuration_json, "max_workers", int
+                ),
                 oncall=ensure_option_type(configuration_json, "oncall", str),
                 other_critical_files=ensure_string_list(
                     configuration_json, "critical_files"
@@ -488,9 +496,6 @@ class PartialConfiguration:
                 unwatched_dependency=unwatched_dependency,
                 use_buck2=ensure_option_type(configuration_json, "use_buck2", bool),
                 version_hash=ensure_option_type(configuration_json, "version", str),
-                use_errpy_parser=ensure_option_type(
-                    configuration_json, "use_errpy_parser", bool
-                ),
             )
 
             # Check for unused keys
@@ -501,17 +506,39 @@ class PartialConfiguration:
 
             return partial_configuration
         except json.JSONDecodeError as error:
-            raise exceptions.InvalidConfiguration("Invalid JSON file") from error
+            raise exceptions.InvalidConfiguration(
+                "Invalid configuration file"
+            ) from error
 
     @staticmethod
     def from_file(path: Path) -> "PartialConfiguration":
+        is_toml = path.suffix.lower() == ".toml"
         try:
             contents = path.read_text(encoding="utf-8")
-            return PartialConfiguration.from_string(contents)
+            if is_toml:
+                configuration_toml = tomllib.loads(contents)
+                configuration_tool = (
+                    configuration_toml["tool"] if "tool" in configuration_toml else {}
+                )
+                configuration_pyre = (
+                    configuration_tool["pyre"] if "pyre" in configuration_tool else {}
+                )
+                return PartialConfiguration.from_dict(configuration_pyre)
+            else:
+                configuration_json = json.loads(contents)
+                return PartialConfiguration.from_dict(configuration_json)
         except OSError as error:
             raise exceptions.InvalidConfiguration(
                 f"Error when reading {path}"
             ) from error
+        except json.JSONDecodeError as error:
+            raise exceptions.InvalidConfiguration(
+                f"Invalid JSON file at {path}"
+            ) from error
+        except tomllib.TOMLDecodeError as error:
+            raise exceptions.InvalidConfiguration(
+                f"Invalid TOML file at {path}"
+            ) from error
 
     def expand_relative_paths(self, root: str) -> "PartialConfiguration":
         unwatched_dependency = self.unwatched_dependency
@@ -588,6 +615,7 @@ class Configuration:
     dot_pyre_directory: Optional[Path] = None
     enable_readonly_analysis: Optional[bool] = None
     enable_strict_override_check: Optional[bool] = None
+    enable_strict_any_check: Optional[bool] = None
     enable_unawaited_awaitable_analysis: Optional[bool] = None
     excludes: Sequence[str] = field(default_factory=list)
     extensions: Sequence[extension.Element] = field(default_factory=list)
@@ -596,6 +624,7 @@ class Configuration:
     isolation_prefix: Optional[str] = None
     logger: Optional[str] = None
     number_of_workers: Optional[int] = None
+    max_number_of_workers: Optional[int] = None
     oncall: Optional[str] = None
     other_critical_files: Sequence[str] = field(default_factory=list)
     pysa_version_hash: Optional[str] = None
@@ -621,7 +650,6 @@ class Configuration:
     unwatched_dependency: Optional[unwatched.UnwatchedDependency] = None
     use_buck2: bool = True
     version_hash: Optional[str] = None
-    use_errpy_parser: bool = False
 
     @staticmethod
     def from_partial_configuration(
@@ -646,6 +674,7 @@ class Configuration:
             ],
             enable_readonly_analysis=partial_configuration.enable_readonly_analysis,
             enable_strict_override_check=partial_configuration.enable_strict_override_check,
+            enable_strict_any_check=partial_configuration.enable_strict_any_check,
             enable_unawaited_awaitable_analysis=(
                 partial_configuration.enable_unawaited_awaitable_analysis
             ),
@@ -659,6 +688,7 @@ class Configuration:
             isolation_prefix=partial_configuration.isolation_prefix,
             logger=partial_configuration.logger,
             number_of_workers=partial_configuration.number_of_workers,
+            max_number_of_workers=partial_configuration.max_number_of_workers,
             oncall=partial_configuration.oncall,
             other_critical_files=partial_configuration.other_critical_files,
             pysa_version_hash=partial_configuration.pysa_version_hash,
@@ -686,9 +716,6 @@ class Configuration:
                 partial_configuration.use_buck2, default=True
             ),
             version_hash=partial_configuration.version_hash,
-            use_errpy_parser=_get_optional_value(
-                partial_configuration.use_errpy_parser, default=False
-            ),
         )
 
     @property
@@ -714,6 +741,7 @@ class Configuration:
         isolation_prefix = self.isolation_prefix
         logger = self.logger
         number_of_workers = self.number_of_workers
+        max_number_of_workers = self.max_number_of_workers
         oncall = self.oncall
         pysa_version_hash = self.pysa_version_hash
         python_version = self.python_version
@@ -742,6 +770,11 @@ class Configuration:
                 if self.enable_strict_override_check is not None
                 else {}
             ),
+            **(
+                {"enable_strict_any_check": self.enable_strict_any_check}
+                if self.enable_strict_any_check is not None
+                else {}
+            ),
             **(
                 {
                     "enable_unawaited_awaitable_analysis": (
@@ -767,6 +800,11 @@ class Configuration:
             **({"logger": logger} if logger is not None else {}),
             **({"oncall": oncall} if oncall is not None else {}),
             **({"workers": number_of_workers} if number_of_workers is not None else {}),
+            **(
+                {"max_workers": max_number_of_workers}
+                if max_number_of_workers is not None
+                else {}
+            ),
             "other_critical_files": list(self.other_critical_files),
             **(
                 {"pysa_version_hash": pysa_version_hash}
@@ -883,11 +921,21 @@ class Configuration:
         if number_of_workers is not None and number_of_workers > 0:
             return number_of_workers
 
-        number_of_physical_cores = psutil.cpu_count(logical=False)
-        if number_of_physical_cores is None:
+        number_of_logical_cores = psutil.cpu_count(logical=True)
+        if number_of_logical_cores is None:
             default_number_of_workers = 1
         else:
-            default_number_of_workers = max(1, number_of_physical_cores - 1)
+            default_number_of_workers = max(1, number_of_logical_cores // 2 - 1)
+            max_number_of_workers = self.max_number_of_workers
+            if (
+                max_number_of_workers is not None
+                and max_number_of_workers > 0
+                and default_number_of_workers > max_number_of_workers
+            ):
+                LOG.info(
+                    f"The number of workers is capped at the maximum {max_number_of_workers}."
+                )
+                default_number_of_workers = max_number_of_workers
 
         LOG.info(
             "Could not determine the number of Pyre workers from configuration. "
@@ -948,8 +996,8 @@ def create_configuration(
     if local_root_argument is not None:
         if found_root is None:
             raise exceptions.InvalidConfiguration(
-                "A local configuration path was explicitly specified, but no"
-                + f" {CONFIGURATION_FILE} file was found in {search_base}"
+                "A local configuration path was explicitly specified, but neither"
+                + f" {JSON_CONFIGURATION_FILE} nor {TOML_CONFIGURATION_FILE} file was found in {search_base}"
                 + " or its parents."
             )
         elif found_root.local_root is None:
@@ -969,9 +1017,17 @@ def create_configuration(
     else:
         project_root = found_root.global_root
         relative_local_root = None
-        partial_configuration = PartialConfiguration.from_file(
-            project_root / CONFIGURATION_FILE
-        ).expand_relative_paths(str(project_root))
+        if (project_root / JSON_CONFIGURATION_FILE).is_file():
+            partial_configuration = PartialConfiguration.from_file(
+                project_root / JSON_CONFIGURATION_FILE
+            ).expand_relative_paths(str(project_root))
+        else:
+            LOG.debug(
+                "Could not find `.pyre_configuration` in the project root.Searching for `pyproject.toml`..."
+            )
+            partial_configuration = PartialConfiguration.from_file(
+                project_root / TOML_CONFIGURATION_FILE
+            ).expand_relative_paths(str(project_root))
         local_root = found_root.local_root
         if local_root is not None:
             relative_local_root = get_relative_local_root(project_root, local_root)
