@@ -24,7 +24,8 @@ import sys
 from pathlib import Path
 from typing import Callable, List, NamedTuple, Optional
 
-CONFIGURATION_FILE: str = ".pyre_configuration"
+JSON_CONFIGURATION_FILE: str = ".pyre_configuration"
+TOML_CONFIGURATION_FILE: str = "pyproject.toml"
 LOCAL_CONFIGURATION_FILE: str = ".pyre_configuration.local"
 BINARY_NAME: str = "pyre.bin"
 CLIENT_NAME: str = "pyre-client"
@@ -122,7 +123,9 @@ def find_outermost_directory_containing_file(
 def find_global_root(base: Path) -> Optional[Path]:
     """Pyre always runs from the directory containing the nearest .pyre_configuration,
     if one exists."""
-    return find_parent_directory_containing_file(base, CONFIGURATION_FILE)
+    return find_parent_directory_containing_file(
+        base, JSON_CONFIGURATION_FILE
+    ) or find_parent_directory_containing_file(base, TOML_CONFIGURATION_FILE)
 
 
 def get_relative_local_root(
@@ -152,7 +155,9 @@ def find_global_and_local_root(base: Path) -> Optional[FoundRoot]:
     return the path to the global configuration.
     If both global and local exist, return them as a pair.
     """
-    found_global_root = find_parent_directory_containing_file(base, CONFIGURATION_FILE)
+    found_global_root = find_parent_directory_containing_file(
+        base, JSON_CONFIGURATION_FILE
+    ) or find_parent_directory_containing_file(base, TOML_CONFIGURATION_FILE)
     if found_global_root is None:
         return None
 
