@@ -32,7 +32,6 @@ from . import (
     command_arguments,
     commands,
     configuration as configuration_module,
-    find_directories,
     frontend_configuration,
     identifiers,
     log,
@@ -315,6 +314,11 @@ def _create_and_check_configuration(
     hidden=True,
 )
 @click.option("--number-of-workers", type=int, help="Number of parallel workers to use")
+@click.option(
+    "--max-number-of-workers",
+    type=int,
+    help="When `--number-of-workers` is not set, this is the maximum number when calculating the default.",
+)
 @click.option("--no-logger", is_flag=True, default=False, hidden=True)
 def pyre(
     context: click.Context,
@@ -345,6 +349,7 @@ def pyre(
     shared_memory_dependency_table_power: Optional[int],
     shared_memory_hash_table_power: Optional[int],
     number_of_workers: Optional[int],
+    max_number_of_workers: Optional[int],
     no_logger: bool,
 ) -> None:
     arguments = command_arguments.CommandArguments(
@@ -384,6 +389,7 @@ def pyre(
         shared_memory_dependency_table_power=shared_memory_dependency_table_power,
         shared_memory_hash_table_power=shared_memory_hash_table_power,
         number_of_workers=number_of_workers,
+        max_number_of_workers=max_number_of_workers,
         use_buck2=None,
     )
     context.ensure_object(dict)
@@ -486,12 +492,6 @@ def pyre(
     default=False,
     help="Build the cache and exit without computing results..",
 )
-@click.option(
-    "--inline-decorators",
-    is_flag=True,
-    default=False,
-    help="Inline decorators at use sites to catch flows through decorators.",
-)
 @click.option(
     "--infer-self-tito",
     is_flag=True,
@@ -589,6 +589,32 @@ def pyre(
     help="Path to a configuration file containing scheduler policies.",
     hidden=True,
 )
+@click.option(
+    "--kill-buck-after-build",
+    is_flag=True,
+    default=False,
+    help="Kill Buck after building the project.",
+)
+@click.option(
+    "--number-of-buck-threads",
+    type=int,
+    help="Limits the number of threads during Buck build.",
+)
+@click.option(
+    "--higher-order-call-graph-max-iterations",
+    type=int,
+    help="Limits the number of fixpoint iterations when building higher order call graphs.",
+)
+@click.option(
+    "--maximum-target-depth",
+    type=int,
+    help="Limits the depth of parameterized targets that are created during higher order call graph building.",
+)
+@click.option(
+    "--maximum-parameterized-targets-at-call-site",
+    type=int,
+    help="Limits the number of parameterized targets that can be created at any call site, during higher order call graph building.",
+)
 @click.pass_context
 def analyze(
     context: click.Context,
@@ -610,7 +636,6 @@ def analyze(
     dump_model_query_results: Optional[str],
     use_cache: bool,
     build_cache_only: bool,
-    inline_decorators: bool,
     infer_self_tito: bool,
     infer_argument_tito: bool,
     maximum_model_source_tree_width: Optional[int],
@@ -629,6 +654,11 @@ def analyze(
     compact_ocaml_heap: bool,
     compute_coverage: bool,
     scheduler_policies: Optional[str],
+    kill_buck_after_build: bool,
+    number_of_buck_threads: Optional[int],
+    higher_order_call_graph_max_iterations: Optional[int],
+    maximum_target_depth: Optional[int],
+    maximum_parameterized_targets_at_call_site: Optional[int],
 ) -> int:
     """
     Run Pysa, the inter-procedural static analysis tool.
@@ -653,7 +683,6 @@ def analyze(
                 if find_missing_flows is not None
                 else None
             ),
-            inline_decorators=inline_decorators,
             infer_self_tito=infer_self_tito,
             infer_argument_tito=infer_argument_tito,
             log_identifier=command_argument.log_identifier,
@@ -695,6 +724,11 @@ def analyze(
             scheduler_policies_path=(
                 Path(scheduler_policies) if scheduler_policies is not None else None
             ),
+            kill_buck_after_build=kill_buck_after_build,
+            number_of_buck_threads=number_of_buck_threads,
+            higher_order_call_graph_max_iterations=higher_order_call_graph_max_iterations,
+            maximum_target_depth=maximum_target_depth,
+            maximum_parameterized_targets_at_call_site=maximum_parameterized_targets_at_call_site,
         ),
     )
 
@@ -825,6 +859,17 @@ def incremental(
         "in a principled way."
     ),
 )
+@click.option(
+    "--kill-buck-after-build",
+    is_flag=True,
+    default=False,
+    help="Kill Buck after building the project.",
+)
+@click.option(
+    "--number-of-buck-threads",
+    type=int,
+    help="Limits the number of threads during Buck build.",
+)
 @click.pass_context
 def infer(
     context: click.Context,
@@ -839,6 +884,8 @@ def infer(
     quote_annotations: bool,
     simple_annotations: bool,
     dequalify: bool,
+    kill_buck_after_build: bool,
+    number_of_buck_threads: Optional[int],
 ) -> int:
     """
     Run pyre infer.
@@ -877,6 +924,8 @@ def infer(
             print_only=print_only,
             read_stdin=read_stdin,
             sequential=command_argument.sequential,
+            kill_buck_after_build=kill_buck_after_build,
+            number_of_buck_threads=number_of_buck_threads,
         ),
     )
 
@@ -1313,6 +1362,11 @@ def servers_stop(context: click.Context) -> int:
         "This is used to disambiguate paths and log handling."
     ),
 )
+@click.option(
+    "--number-of-buck-threads",
+    type=int,
+    help="Limits the number of threads during Buck build.",
+)
 @click.pass_context
 def start(
     context: click.Context,
@@ -1324,6 +1378,7 @@ def start(
     use_lazy_module_tracking: bool,
     analyze_external_sources: bool,
     flavor: Optional[str],
+    number_of_buck_threads: Optional[int],
 ) -> int:
     """
     Starts a pyre server as a daemon_socket.
@@ -1346,6 +1401,7 @@ def start(
             skip_initial_type_check=skip_initial_type_check,
             use_lazy_module_tracking=use_lazy_module_tracking,
             analyze_external_sources=analyze_external_sources,
+            number_of_buck_threads=number_of_buck_threads,
         ),
     )
 
